import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

import 'senior_elevated_element.dart';

enum SeniorButtonSize { small, big }
enum SeniorButtonType { primary, secondary, ghost }

abstract class SeniorButtonTheme {
  static const double minWidth = 156.0;
  static const double maxWidth = 328.0;

  static double getButtonWidth({required double width}) {
    if (width < minWidth) {
      return minWidth;
    }
    if (width > maxWidth) {
      return maxWidth;
    }
    return width;
  } 

  static double getButtonHeight({required SeniorButtonSize size}) {
    switch (size) {
      case SeniorButtonSize.big:
        return 48.0;
      case SeniorButtonSize.small:
      default:
        return 40.0;
    }
  }

  static Color _getButtonColor({
    required Brightness theme,
    required SeniorButtonType type,
    required bool isDisabled
  }) {
    switch (type) {
      case SeniorButtonType.ghost:
        return Colors.transparent;
      case SeniorButtonType.secondary:
        return !isDisabled
            ? SeniorColors.secondaryColor600
            : SeniorColors.secondaryColor100;
      case SeniorButtonType.primary:
      default:
        return !isDisabled
            ? SeniorColors.primaryColor
            : SeniorColors.primaryColor100;
    }
  }

  static Color _getButtonHighlightColor({
    required Brightness theme,
    required SeniorButtonType type
  }) {
    switch (type) {
      case SeniorButtonType.ghost:
        return SeniorColors.neutralColor400;
      case SeniorButtonType.secondary:
        return SeniorColors.secondaryColor100;
      case SeniorButtonType.primary:
      default:
        return SeniorColors.primaryColor100;
    }
  }

  static Color getButtonTextColor({
    required Brightness theme,
    required SeniorButtonType type,
    required bool isDisabled
  }) {
    switch (type) {
      case SeniorButtonType.ghost:
        return !isDisabled
            ? SeniorColors.primaryColor
            : SeniorColors.primaryColor100;
      case SeniorButtonType.secondary:
        return !isDisabled
            ? SeniorColors.pureWhite
            : SeniorColors.secondaryColor400;
      case SeniorButtonType.primary:
      default:
        return !isDisabled
            ? SeniorColors.pureWhite
            : SeniorColors.primaryColor300;
    }
  }

  static TextStyle getButtonTextStyle({
    required Brightness theme,
    required SeniorButtonType type,
    required bool isDisabled
  }) {
    return SeniorTypography.getTextStyle(
      token: SeniorTypographyTokens.CTA,
      color: getButtonTextColor(theme: theme, type: type, isDisabled: isDisabled),
    );
  }

  static ButtonStyle getButtonStyle({
    required Brightness theme,
    required SeniorButtonType type,
    required bool isDisabled,
  }) {
    return ButtonStyle(
      overlayColor: MaterialStateProperty.all<Color>(_getButtonHighlightColor(
        theme: theme,
        type: type,
      )),
      backgroundColor: MaterialStateProperty.all<Color>(_getButtonColor(
        theme: theme,
        type: type,
        isDisabled: isDisabled,
      )),
      shape: MaterialStateProperty.all<OutlinedBorder>(
        new RoundedRectangleBorder(
          borderRadius: new BorderRadius.circular(SeniorRadius.xbig),
        ),
      ),
      //A elevação é configurada pelo SeniorElevatedElement
      elevation: MaterialStateProperty.all<double>(0),
    );
  }
}

class SeniorButton extends StatelessWidget {
  final String label;
  final bool disabled;
  final VoidCallback onPressed;
  final SeniorButtonType type;
  final double width;
  final SeniorButtonSize size;

  const SeniorButton({
    required Key key,
    required this.label,
    this.disabled = false,
    required this.onPressed,
    this.type = SeniorButtonType.primary,
    this.width = SeniorButtonTheme.maxWidth,
    this.size = SeniorButtonSize.small,
  }) : super(key: key);

  factory SeniorButton.icon({
    required Key key,
    required String label,
    bool disabled,
    required VoidCallback onPressed,
    required IconData icon,
    SeniorButtonType type,
    double width,
    SeniorButtonSize size,
  }) = _SeniorButtonIcon;

  Widget _buildButton({ required Brightness theme }) {
    return SeniorElevatedElement(
      child: ElevatedButton(
        key: this.key,
        child: Text(
          this.label,
          style: SeniorButtonTheme.getButtonTextStyle(theme: theme, type: this.type, isDisabled: this.disabled),
        ),
        onPressed: !this.disabled ? this.onPressed : null,
        style: SeniorButtonTheme.getButtonStyle(theme: theme, type: this.type, isDisabled: this.disabled),
      ),
      elevation: this.disabled || this.type == SeniorButtonType.ghost
          ? SeniorElevations.dp0
          : SeniorElevations.dp01,
      borderRadius: SeniorRadius.xbig,
    );
  }

  @override
  Widget build(BuildContext context) {
    Brightness theme = MediaQuery.of(context).platformBrightness;

    return SizedBox(
      child: this._buildButton(theme: theme),
      height: SeniorButtonTheme.getButtonHeight(size: this.size),
      width: SeniorButtonTheme.getButtonWidth(width: this.width),
    );
  }
}

class _SeniorButtonIcon extends SeniorButton {
  final IconData icon;

  _SeniorButtonIcon({
    required Key key,
    required String label,
    bool disabled = false,
    required VoidCallback onPressed,
    required this.icon,
    SeniorButtonType type = SeniorButtonType.primary,
    double width = SeniorButtonTheme.maxWidth,
    SeniorButtonSize size = SeniorButtonSize.small,
  }) : super(
          key: key,
          label: label,
          disabled: disabled,
          onPressed: onPressed,
          type: type,
          width: width,
          size: size,
        );

  @override
  Widget _buildButton({ required Brightness theme }) {
    return SeniorElevatedElement(
      child: ElevatedButton.icon(
        label: Text(
          this.label,
          style: SeniorButtonTheme.getButtonTextStyle(
            theme: theme,
            type: this.type,
            isDisabled: this.disabled,
          ),
        ),
        icon: FaIcon(
          this.icon,
          size: SeniorTypographyIconSize.CTA,
          color: SeniorButtonTheme.getButtonTextColor(
            theme: theme,
            type: this.type,
            isDisabled: this.disabled,
          ),
        ),
        onPressed: !this.disabled ? this.onPressed : null,
        style: SeniorButtonTheme.getButtonStyle(
          theme: theme,
          type: this.type,
          isDisabled: this.disabled,
        ),
      ),
      elevation: this.disabled || this.type == SeniorButtonType.ghost
          ? SeniorElevations.dp0
          : SeniorElevations.dp01,
      borderRadius: SeniorRadius.xbig,
    );
  }
}
