import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';
import 'package:senior_design_tokens/tokens/senior_typography.dart';

abstract class SeniorTextFieldTheme {
  static Color getFillColor({ required Brightness theme }) {
    return SeniorColors.neutralColor100; 
  }

  static Color getBorderColor({
    required Brightness theme,
    required bool hasFocus,
    required bool hasError
  }) {
    if (hasFocus) {
      if (hasError) {
        return SeniorColors.manchesterColorRed;
      }
      return SeniorColors.primaryColor;
    }
    return Colors.transparent;
  }

  static Color getHelperTextColor({
    required Brightness theme,
    required bool hasError
  }) {
    if (hasError) {
      return SeniorColors.manchesterColorRed;
    }
    return SeniorColors.neutralColor700;
  }

  static BorderRadius getBorderRadiusValues() {
    return BorderRadius.only(
      topLeft: Radius.circular(SeniorRadius.xbig),
      topRight: Radius.circular(SeniorRadius.xbig),
      bottomRight: Radius.circular(SeniorRadius.xsmall),
      bottomLeft: Radius.circular(SeniorRadius.xsmall),
    );
  }

  static TextStyle getTextStyle({required Brightness theme}) {
    return SeniorTypography.getTextStyle(
      token: SeniorTypographyTokens.text,
      color: SeniorColors.neutralColor900,
    );
  }

  static TextStyle getHintTextStyle({required Brightness theme}) {
    return SeniorTypography.getTextStyle(
      token: SeniorTypographyTokens.text,
      color: SeniorColors.neutralColor600,
    );
  }

  static TextStyle getLabelStyle({
    required Brightness theme,
    required bool hasFocus,
    required bool hasError
  }) {
    return TextStyle(
      height: hasFocus && !hasError ? 3 : 0,
      color: hasError
        ? SeniorColors.manchesterColorRed
        : SeniorColors.primaryColor500,
      fontSize: 12.0,
      fontWeight: FontWeight.w400,
      fontFamily: 'OpenSans',
    );
  }
}

class SeniorTextField extends StatefulWidget {
  final String label;
  final IconData? leftIcon;
  final IconData? rightIcon;
  final String helper;
  final String? hintText;
  final bool disabled;
  final String? initialValue;
  final FormFieldValidator<String>? validator;
  final int? maxLength;
  final int? minLines;
  final int? maxLines;
  final bool autocorrect;
  final bool readOnly;
  final Function(String)? onChange;
  final Function()? onTap;
  final Function(String?)? onSaved;
  final Function()? onEditingComplete;
  final Function(String)? onFieldSubmitted;
  final bool autofocus;
  final AutovalidateMode? autovalidateMode;
  final TextEditingController? controller;

  SeniorTextField({
    required Key key,
    this.label = '',
    this.leftIcon,
    this.rightIcon,
    this.helper = '',
    this.hintText,
    this.disabled = false,
    this.initialValue,
    this.validator,
    this.maxLength,
    this.minLines,
    this.maxLines,
    this.autocorrect = true,
    this.readOnly = false,
    this.onChange,
    this.onTap,
    this.onSaved,
    this.onEditingComplete,
    this.onFieldSubmitted,
    this.autofocus = false,
    this.autovalidateMode,
    this.controller,
  }) : super(key: key);

  @override
  _SeniorTextField createState() => _SeniorTextField();
}

class _SeniorTextField extends State<SeniorTextField> {
  late FocusNode _focus;
  bool _hasFocus = false;
  String? validMsg;

  @override
  void initState() {
    super.initState();
    _focus = FocusNode();
    _focus.addListener(_onFocusChange);
  }

  @override
  void dispose() {
    _focus.dispose();
    super.dispose();
  }

  void _onFocusChange() {
    setState(() {
      _hasFocus = _focus.hasFocus;
    });
  }

  String? _validator(String? value) {
    if (widget.validator == null) return null;

    String? msg = widget.validator!(value);

    setState(() {
      this.validMsg = msg != null && msg.isNotEmpty ? msg : null;
    });

    return validMsg;
  }

  Widget? _buildFaIcon(IconData? icon) {
    if (icon != null) {
      return Padding(
        padding: const EdgeInsets.symmetric(
          horizontal: 12.0,
          vertical: 16.0,
        ),
        child: FaIcon(
          icon,
          size: SeniorTypographyIconSize.CTA,
          color: SeniorColors.neutralColor900,
        ),
      );
    }
    return null;
  }

  Widget _buildTextField({required Brightness theme}) {
    return Container(
      decoration: BoxDecoration(
        borderRadius: SeniorTextFieldTheme.getBorderRadiusValues(),
        border: Border.all(
          color: SeniorTextFieldTheme.getBorderColor(
            theme: theme,
            hasFocus: _hasFocus,
            hasError: this.validMsg != null,
          ),
          width: 2.0,
        ),
      ),
      child: TextFormField(
        key: widget.key,
        style: SeniorTextFieldTheme.getTextStyle(theme: theme),
        focusNode: _focus,
        maxLength: widget.maxLength,
        minLines: widget.minLines,
        maxLines: widget.maxLines,
        autocorrect: widget.autocorrect,
        readOnly: widget.readOnly,
        onChanged: widget.onChange,
        onTap: widget.onTap,
        onSaved: widget.onSaved,
        onEditingComplete: widget.onEditingComplete,
        onFieldSubmitted: widget.onFieldSubmitted,
        validator: this._validator,
        initialValue: widget.initialValue,
        autofocus: widget.autofocus,
        enabled: !widget.disabled,
        autovalidateMode: widget.autovalidateMode,
        controller: widget.controller,
        decoration: InputDecoration(
          labelText: widget.label,
          labelStyle: SeniorTextFieldTheme.getLabelStyle(
            theme: theme,
            hasFocus: _hasFocus,
            hasError: validMsg != null,
          ),
          prefixIcon: this._buildFaIcon(widget.leftIcon),
          suffixIcon: this._buildFaIcon(widget.rightIcon),
          filled: true,
          fillColor: SeniorTextFieldTheme.getFillColor(theme: theme),
          focusedBorder: OutlineInputBorder(
            borderSide: BorderSide(style: BorderStyle.none),
          ),
          errorStyle: TextStyle(
            fontSize: 0,
            height: 0,
          ),
          focusedErrorBorder: InputBorder.none,
          hintText: widget.hintText,
          hintStyle: SeniorTextFieldTheme.getHintTextStyle(theme: theme),
        ),
      ),
    );
  }

  Widget _buildHelper({required Brightness theme}) {
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 14.0, vertical: 4.0),
      child: Text(
        this.validMsg ?? widget.helper,
        style: SeniorTypography.getTextStyle(
          token: SeniorTypographyTokens.helper,
          color: SeniorTextFieldTheme.getHelperTextColor(
            theme: theme,
            hasError: this.validMsg != null,
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    Brightness theme = MediaQuery.of(context).platformBrightness;

    return Column(
      children: [
        this._buildTextField(theme: theme),
        this._buildHelper(theme: theme),
      ],
      crossAxisAlignment: CrossAxisAlignment.start,
    );
  }
}
